`timescale 1ns/100ps
module RAM18x512_test(Clock, Resetn, Compare_ok);
input Clock, Resetn;
output Compare_ok;
parameter ADDRESS_MSB = 8; 
parameter WIDTH = 18;
parameter TOGGLE1 = 18'b101010101010101010,
	  TOGGLE2 = 18'b111111111111111111;
reg  [WIDTH-1:0] ram18x512_d,ram18x512_c;
wire [WIDTH-1:0] wr_chk_d,wr_inc_d,wr_gry_d,wr_alt_d;
wire [WIDTH-1:0] ram18x512_q,rd_chk_d,rd_inc_d,rd_gry_d,rd_alt_d;

wire [ADDRESS_MSB:0] wr_address, rd_address;
//wire [WIDTH-1:0] RAM18x512_d,RAM18x512_q;
wire [1:0] pattern_select;
wire cmp_en, wr_en, rd_en;

reg Compare_ok;
// Install RAM BIST controller for depth of 64
RAMBIST_FSM #(ADDRESS_MSB) 
 UFSM18x512(.Clock(Clock),  
	  .Resetn(Resetn), 
	  .rd_address(rd_address), 
	  .wr_address(wr_address), 
	  .pattern_select(pattern_select), 
          .wr_en(wr_en), 
	  .rd_en(rd_en), 
	  .cmp_en(cmp_en)
	 );

counter #(ADDRESS_MSB+1) URDADD(.clk(Clock),.aclr(Resetn),.ena(rd_en),.count(rd_address));
counter #(ADDRESS_MSB+1) UWRADD(.clk(Clock),.aclr(Resetn),.ena(wr_en),.count(wr_address));

RAM18x512 URAM18x512(.WAddress(wr_address),
	         .RAddress(rd_address),
		 .Data(ram18x512_d),
		 .WClock(Clock),
		 .RClock(Clock),
		 .WE(wr_en),
		 .RE(rd_en),
		 .Q(ram18x512_q)
	        );

parameter //Define the different RAM pattern types
CHECKERS  = 0, // 00 pattern is 0101 then 1010
INCREMENT = 1, // 01 pattern is incrementing pattern
GRAY      = 2, // 10 pattern is gray code pattern
ALTERNATE = 3; // 11 pattern is 1111 then 0000

wire wr_chk_en = wr_en && pattern_select==CHECKERS;
wire wr_inc_en = wr_en && pattern_select==INCREMENT;
wire wr_gry_en = wr_en && pattern_select==GRAY;
wire wr_alt_en = wr_en && pattern_select==ALTERNATE;

toggle       #(WIDTH,TOGGLE1) UWRCHK(.clk(Clock),.aclr(Resetn),.ena(wr_chk_en),.toggle(wr_chk_d));
counter      #(WIDTH)       UWRINC(.clk(Clock),.aclr(Resetn),.ena(wr_inc_en),.count(wr_inc_d));
gray_counter #(WIDTH)       UWRGRY(.clk(Clock),.aclr(Resetn),.ena(wr_gry_en),.gray_code(wr_gry_d));
toggle       #(WIDTH,TOGGLE2) UWRALT(.clk(Clock),.aclr(Resetn),.ena(wr_alt_en),.toggle(wr_alt_d));

always @(pattern_select or wr_chk_d or wr_inc_d or wr_gry_d or wr_alt_d)
 case (pattern_select)
  CHECKERS:	ram18x512_d = wr_chk_d;
  INCREMENT:	ram18x512_d = wr_inc_d;
  GRAY:         ram18x512_d = wr_gry_d;
  ALTERNATE:    ram18x512_d = wr_alt_d;
 endcase

wire cmp_chk_en = cmp_en && pattern_select==CHECKERS;
wire cmp_inc_en = cmp_en && pattern_select==INCREMENT;
wire cmp_gry_en = cmp_en && pattern_select==GRAY;
wire cmp_alt_en = cmp_en && pattern_select==ALTERNATE;
toggle       #(WIDTH,TOGGLE1) URDCHK(.clk(Clock),.aclr(Resetn),.ena(cmp_chk_en),.toggle(rd_chk_d));
counter      #(WIDTH)       URDINC(.clk(Clock),.aclr(Resetn),.ena(cmp_inc_en),.count(rd_inc_d));
gray_counter #(WIDTH)       URDGRY(.clk(Clock),.aclr(Resetn),.ena(cmp_gry_en),.gray_code(rd_gry_d));
toggle       #(WIDTH,TOGGLE2) URDALT(.clk(Clock),.aclr(Resetn),.ena(cmp_alt_en),.toggle(rd_alt_d));

always @(pattern_select or rd_chk_d or rd_inc_d or rd_gry_d or rd_alt_d)
 case (pattern_select)
  CHECKERS:	ram18x512_c = rd_chk_d;
  INCREMENT:	ram18x512_c = rd_inc_d;
  GRAY:         ram18x512_c = rd_gry_d;
  ALTERNATE:    ram18x512_c = rd_alt_d;
 endcase
always @(posedge Clock or negedge Resetn)
 if (Resetn==1'b0) 
  Compare_ok = 1'b1;
 else
  Compare_ok = !cmp_en || (ram18x512_c == ram18x512_q);

endmodule
