`timescale 1ns/100ps
// 8 IO Group
// Manish R. Babladi
// 03/05/2003
// 
module io_test(Clock, Resetn, Oe, Test_ok, Io_pad)/* synthesis syn_hier = firm */;

parameter WIDTH = 8,
          DEPTH = 8; // Caution DEPTH parameter must be mod 2.
  
input Clock, Resetn, Oe;
output Test_ok; 
reg Test_ok;
inout [WIDTH-1:0] Io_pad;

wire [WIDTH-1:0] Io_pad_y;
wire [WIDTH-1:0] Io_pad_d;
/*
assign Io_pad_y = Io_pad; //Will be replace once BIBUFs are added.
assign Io_pad = (1'b1) ? Io_pad_d : 4'bzzzz; //Caution doesn't use WIDTH parameter
*/
/* 
 * Note that without a Generate statement this section cannot
 * automatically handle the DEPTH parameter
 */
BIBUF UIO_0(.PAD(Io_pad[0]),.E(Oe),.D(Io_pad_d[0]),.Y(Io_pad_y[0]));
BIBUF UIO_1(.PAD(Io_pad[1]),.E(Oe),.D(Io_pad_d[1]),.Y(Io_pad_y[1]));
BIBUF UIO_2(.PAD(Io_pad[2]),.E(Oe),.D(Io_pad_d[2]),.Y(Io_pad_y[2]));
BIBUF UIO_3(.PAD(Io_pad[3]),.E(Oe),.D(Io_pad_d[3]),.Y(Io_pad_y[3]));
BIBUF UIO_4(.PAD(Io_pad[4]),.E(Oe),.D(Io_pad_d[4]),.Y(Io_pad_y[4]));
BIBUF UIO_5(.PAD(Io_pad[5]),.E(Oe),.D(Io_pad_d[5]),.Y(Io_pad_y[5]));
BIBUF UIO_6(.PAD(Io_pad[6]),.E(Oe),.D(Io_pad_d[6]),.Y(Io_pad_y[6]));
BIBUF UIO_7(.PAD(Io_pad[7]),.E(Oe),.D(Io_pad_d[7]),.Y(Io_pad_y[7]));

wire [WIDTH-1:0] gray_write;
reg [WIDTH-1:0] gray_check_reg;
reg [WIDTH-1:0] Io_pad_y_reg;
reg [WIDTH-1:0] gray_check;
reg test_reg;

// Caution DEPTH parameter must be mod 2.
wire [(DEPTH/2)-1:0] j_i;

sr_stack #(DEPTH,WIDTH) USRSTK(.Clock(Clock), .Resetn(Resetn), .Enable(1'b1), .Di(gray_write), .Do(Io_pad_d));

// Caution DEPTH parameter must be mod 2.
johnson_counter #(DEPTH/2) UJOHN_I(.clk(Clock), .aclr(Resetn), .ena(1'b1), .johnson_code(j_i));
wire inc_gray = j_i==0;

gray_counter #(WIDTH) UGRAY_I(.clk(Clock), .aclr(Resetn), .ena(inc_gray), .gray_code(gray_write));

always @(posedge Clock or negedge Resetn)
 if (Resetn==1'b0) begin
  test_reg <= 1'b0;
  gray_check <= 0;
  gray_check_reg <= 0;
  Io_pad_y_reg <= 0;
 end
 else begin
  test_reg <= gray_check_reg==Io_pad_y_reg;
  if (inc_gray) gray_check <= gray_write;
  gray_check_reg <= gray_check;
  Io_pad_y_reg <= Io_pad_y;
 end

/* Optionally this R-Cell can be changed to a CC-Flop to reduce R-Cell
 * count, but the declaration of Test_ok, must be changed from "reg" to
 * "wire" above.
 */
//DFP1B_CC UTOK(.D(1'b0),.CLK(!test_reg),.PRE(Resetn), .Q(Test_ok));
always @(negedge test_reg or negedge Resetn)
 if (Resetn==1'b0)
  Test_ok <= 1'b1;
 else
  Test_ok <= 1'b0;

endmodule
