`timescale 1 ns/100 ps

// RAMLATCH BIST MODEL
// Manish R. Babladi
// 02/03/2003

module ram8x16(Clk, WR, Wadd, Radd, DIN, DO)/*synthesis syn_hier = "hard" */;
parameter DEPTH_LOG2 = 4;
input Clk, WR;
input[7:0] DIN;
input[DEPTH_LOG2 - 1:0] Wadd, Radd;
output[7:0] DO;

wire[7:0] DO0,DO1,DO2,DO3,DO4,DO5,DO6,DO7,
          DO8,DO9,DO10,DO11,DO12,DO13,DO14,DO15,
		  DO_lo, DO_hi;
reg[15:0] WRENABLE;

// add logic to generate WRENABLE signals

always@(WR or Wadd)
  case (Wadd)
   4'd0:  if (WR == 1) WRENABLE= 16'b1111_1111_1111_1110; else WRENABLE= 16'b1111_1111_1111_1111;
   4'd1:  if (WR == 1) WRENABLE= 16'b1111_1111_1111_1101; else WRENABLE= 16'b1111_1111_1111_1111;
   4'd2:  if (WR == 1) WRENABLE= 16'b1111_1111_1111_1011; else WRENABLE= 16'b1111_1111_1111_1111;
   4'd3:  if (WR == 1) WRENABLE= 16'b1111_1111_1111_0111; else WRENABLE= 16'b1111_1111_1111_1111;
   4'd4:  if (WR == 1) WRENABLE= 16'b1111_1111_1110_1111; else WRENABLE= 16'b1111_1111_1111_1111;
   4'd5:  if (WR == 1) WRENABLE= 16'b1111_1111_1101_1111; else WRENABLE= 16'b1111_1111_1111_1111;
   4'd6:  if (WR == 1) WRENABLE= 16'b1111_1111_1011_1111; else WRENABLE= 16'b1111_1111_1111_1111;
   4'd7:  if (WR == 1) WRENABLE= 16'b1111_1111_0111_1111; else WRENABLE= 16'b1111_1111_1111_1111;
   4'd8:  if (WR == 1) WRENABLE= 16'b1111_1110_1111_1111; else WRENABLE= 16'b1111_1111_1111_1111;
   4'd9:  if (WR == 1) WRENABLE= 16'b1111_1101_1111_1111; else WRENABLE= 16'b1111_1111_1111_1111;
   4'd10: if (WR == 1) WRENABLE= 16'b1111_1011_1111_1111; else WRENABLE= 16'b1111_1111_1111_1111;
   4'd11: if (WR == 1) WRENABLE= 16'b1111_0111_1111_1111; else WRENABLE= 16'b1111_1111_1111_1111;
   4'd12: if (WR == 1) WRENABLE= 16'b1110_1111_1111_1111; else WRENABLE= 16'b1111_1111_1111_1111;
   4'd13: if (WR == 1) WRENABLE= 16'b1101_1111_1111_1111; else WRENABLE= 16'b1111_1111_1111_1111;
   4'd14: if (WR == 1) WRENABLE= 16'b1011_1111_1111_1111; else WRENABLE= 16'b1111_1111_1111_1111;
   4'd15: if (WR == 1) WRENABLE= 16'b0111_1111_1111_1111; else WRENABLE= 16'b1111_1111_1111_1111;
   default: WRENABLE= 16'b1111_1111_1111_1111;
  endcase

reg8latch UR0 (.DATA(DIN), .ENABLE(WRENABLE[0]),  .CLOCK(Clk), .Q(DO0));
reg8latch UR1 (.DATA(DIN), .ENABLE(WRENABLE[1]),  .CLOCK(Clk), .Q(DO1));
reg8latch UR2 (.DATA(DIN), .ENABLE(WRENABLE[2]),  .CLOCK(Clk), .Q(DO2));
reg8latch UR3 (.DATA(DIN), .ENABLE(WRENABLE[3]),  .CLOCK(Clk), .Q(DO3));
reg8latch UR4 (.DATA(DIN), .ENABLE(WRENABLE[4]),  .CLOCK(Clk), .Q(DO4));
reg8latch UR5 (.DATA(DIN), .ENABLE(WRENABLE[5]),  .CLOCK(Clk), .Q(DO5));
reg8latch UR6 (.DATA(DIN), .ENABLE(WRENABLE[6]),  .CLOCK(Clk), .Q(DO6));
reg8latch UR7 (.DATA(DIN), .ENABLE(WRENABLE[7]),  .CLOCK(Clk), .Q(DO7));
reg8latch UR8 (.DATA(DIN), .ENABLE(WRENABLE[8]),  .CLOCK(Clk), .Q(DO8));
reg8latch UR9 (.DATA(DIN), .ENABLE(WRENABLE[9]),  .CLOCK(Clk), .Q(DO9));
reg8latch UR10(.DATA(DIN), .ENABLE(WRENABLE[10]), .CLOCK(Clk), .Q(DO10));
reg8latch UR11(.DATA(DIN), .ENABLE(WRENABLE[11]), .CLOCK(Clk), .Q(DO11));
reg8latch UR12(.DATA(DIN), .ENABLE(WRENABLE[12]), .CLOCK(Clk), .Q(DO12));
reg8latch UR13(.DATA(DIN), .ENABLE(WRENABLE[13]), .CLOCK(Clk), .Q(DO13));
reg8latch UR14(.DATA(DIN), .ENABLE(WRENABLE[14]), .CLOCK(Clk), .Q(DO14));
reg8latch UR15(.DATA(DIN), .ENABLE(WRENABLE[15]), .CLOCK(Clk), .Q(DO15));

mux8x8 UM0(.Data0(DO0),.Data1(DO1),.Data2(DO2),.Data3(DO3),.Data4(DO4),.Data5(DO5),.Data6(DO6),.Data7(DO7),
           .Sel(Radd[DEPTH_LOG2-2:0]),.Result(DO_lo));
mux8x8 UM1(.Data0(DO8),.Data1(DO9),.Data2(DO10),.Data3(DO11),.Data4(DO12),.Data5(DO13),.Data6(DO14),.Data7(DO15),
           .Sel(Radd[DEPTH_LOG2-2:0]),.Result(DO_hi));
/*  
 *  2-1 Multiplexor to select the high RAM bank or the low RAM bank
 */
assign DO = (Radd[DEPTH_LOG2-1]) ? DO_hi : DO_lo;

endmodule
