`timescale 1 ns/100 ps

//RAMLATCH BIST MODEL
//Manish R. Babladi
//02/03/2003

module ramlatch (Clock, WR, RD, Radd, Wadd, DI, Resetn, DO);

parameter DEPTH_LOG2 = 4;

input Clock;
input WR, RD; // Write/Read Enables
input[7:0] DI; // Data into the Reg
input [DEPTH_LOG2 -1:0] Wadd, Radd; // Write/Read Addresses
input Resetn;
output[7:0] DO; // Data out

/*
 * For 16 deep set DEPTH_LOG2 to 4
 * For 8 deep set DEPTH_LOG2 to 3
 */

wire[DEPTH_LOG2 - 1:0] Radd; // Read Address
wire[DEPTH_LOG2 - 1:0] Wadd; //Write Address
wire[7:0] DIR; // Data Input Register
wire[7:0] ram_do; //RAM Data Out
wire RD; //Read RAM
wire WR; //Write RAM

/*
 * For 8x8 instantiate ram8x8
 * For 8x16 instantiate ram8x16
 */
// RAM Latch

ram8x16 URAM(.Clk(Clock), .WR(WR), .Wadd(Wadd), .Radd(Radd), .DIN(DIR), .DO(ram_do));

// Data Input
reg8e UIR(.Clock(Clock), .Data(DI), .Q(DIR), .Enable(WR));

// Data output
reg8ec UOR(.Clock(Clock), .Data(ram_do), .Aclr(Resetn), .Q(DO), .Enable(RD));

endmodule
