`timescale 1 ns/100 ps

// RAMLATCH BIST MODEL
// Manish R. Babladi
// 02/03/2003

module test_ramlatch(Clock, Resetn, Wr, Rd, Radd, Wadd, Rd_data, Wr_data, Test_ok);

parameter DEPTH_LOG2 = 4;

input Clock, Resetn;
input [7:0] Rd_data; 
output [7:0] Wr_data;
output  Wr, Rd, Test_ok;
output [DEPTH_LOG2 - 1:0] Radd; // Read Address
output [DEPTH_LOG2 - 1:0] Wadd; //Write Address

reg [3:0] reset_delay;
wire resetn_sync;

reg test_reg;
reg Test_ok;

reg write_enable;
/*reg wr_enable_next;*/
wire Wr_next;

reg read_enable;
//wire read_enable_next;
/*reg rd_enable_next;*/
wire Rd_next;

wire [7:0] check_data;
//wire next_wr_gray, next_rd_gray;
wire Wr1, Compare_Enable;

assign Rd_next = read_enable;
DFC1B URDO(.D(Rd_next), .CLK(Clock), .CLR(resetn_sync), .Q(Rd));

assign Wr_next = write_enable;
DFC1B UWRO(.D(Wr_next), .CLK(Clock), .CLR(resetn_sync), .Q(Wr));
DFC1B UWR1(.D(Wr), .CLK(Clock), .CLR(resetn_sync), .Q(Wr1));
DFC1B UWR2(.D(Wr1), .CLK(Clock), .CLR(resetn_sync), .Q(Compare_Enable));

// Added RADD and WADD counters for RAM LATCH -

// Read Address counter
ptr_counter #(DEPTH_LOG2) URAD(.Enable(Rd), .Clock(Clock), .Aclr(resetn_sync), .Q(Radd));

//Write Address counter
ptr_counter #(DEPTH_LOG2) UWAD(.Enable(Wr1), .Clock(Clock), .Aclr(resetn_sync), .Q(Wadd));

/*gray_counter #(8) UWRGRAY(.clk(Clock), .aclr(Resetn), .ena(write_enable), .gray_code(Wr_data));*/
gray_counter #(8) UWRGRAY(.clk(Clock), .aclr(resetn_sync), .ena(Wr), .gray_code(Wr_data));

/*gray_counter #(8) URDGRAY(.clk(Clock), .aclr(Resetn), .ena(next_rd_gray), .gray_code(check_data));*/
gray_counter #(8) URDGRAY(.clk(Clock), .aclr(resetn_sync), .ena(Compare_Enable), .gray_code(check_data));

always @(posedge Clock or negedge Resetn)
begin: URST
  if (!Resetn)
   reset_delay <= 4'b0;
  else
   reset_delay <= {reset_delay[3:0], 1'b1};       // shift left 1 place; lsb = 1
end // URST;

assign resetn_sync = reset_delay[2];

wire data_err = Compare_Enable && check_data!=Rd_data;

always @(posedge Clock or negedge Resetn) 
begin: UTERR
 if (Resetn==1'b0)
   test_reg <= 0;
 else
  test_reg <= !data_err;
end //UTERR;

always @(posedge Clock or negedge resetn_sync) 
begin: USEQ
 if (resetn_sync==1'b0) begin
  write_enable <= 1'b0;
  read_enable <= 1'b0;
 end
 else begin
  write_enable <= 1'b1;
  read_enable  <= 1'b1;
/*    read_enable  <= 1'b0;*/

 end
end //USEQ;

always @(negedge test_reg or negedge Resetn)
 if (Resetn==1'b0)
  Test_ok <= 1'b1;
 else
  Test_ok <= 1'b0;

endmodule
