`timescale 1 ns/100 ps

//FIFO Test FSM
// Watchdog is removed
// Manish R. Babladi
// 08/10/02
//
module testfifo(Clock, Resetn, FFn, AFn, EFn, AEn, Rd_data, Wr_data, Wrn, Rdn, Test_ok);
parameter WIDTH = 8;
parameter COUNT_WIDTH = 17;
input Clock, Resetn, FFn, AFn, EFn, AEn;
input [WIDTH - 1:0] Rd_data;
output [WIDTH - 1:0] Wr_data;
output  Wrn, Rdn, Test_ok;

reg [4:0] reset_delay;
wire resetn_sync;
reg start_test;
//attribute syn_preserve : boolean;
//attribute syn_encoding : string;

reg T1,T2,T3;
reg test_reg;
reg Test_ok;
//wire write_timeout,read_timeout;

reg write_enable;
reg wr_enable_next;
wire Wrn_next;

reg read_enable;
reg rd_enable_next;
wire Rdn_next;

parameter [1:0] Test1=0, Test2=1, Test3=2, Test4=3;
reg [1:0] test_state /* synthesis syn_encoding = "sequential" */;

parameter [1:0] WRITE_TILL_FULL=0, WRITE_ONE_THIRD=1, WRITE_BURST=2, DISABLE_WRITE=3;
reg [1:0] write_mode /* synthesis syn_encoding = "sequential" */;

parameter [1:0] READ_TILL_EMPTY=0, READ_ONE_THIRD=1, READ_BURST=2, READ_IF_DATA=3;
reg [1:0] read_mode /* synthesis syn_encoding = "sequential" */;

wire [2:0] READ_BURST_cnt, WRITE_BURST_cnt;

reg rbursting,rbursting_next;
reg wbursting,wbursting_next;
wire [2:0] burst_load;
 
wire load_write_count;
wire [COUNT_WIDTH - 1:0] write_count;
wire [COUNT_WIDTH - 1:0] load_value;

reg write_done;
reg test_done;
reg En_read_till_empty;
reg En_write_till_full;

wire [7:0] wr_john, rd_john;
/*wire [WIDTH - 1:0] check_data;*/
// following 2 statements modified from above
reg [WIDTH - 1:0] check_data;
wire [WIDTH - 1:0] check_data_d;
wire next_wr_gray, next_rd_gray;

/*johnson_counter #(8) UWRJOHN(.clk(Clock), .aclr(Resetn), .ena(write_enable), .johnson_code(wr_john));*/
/*modified from above*/
johnson_counter #(8) UWRJOHN(.clk(Clock), .aclr(Resetn), .ena(write_enable && FFn), .johnson_code(wr_john));
assign next_wr_gray = write_enable && wr_john[7] && !wr_john[6] && FFn; //Johnson Code WIDTH'b1000_0000 simplified decode
gray_counter #(WIDTH) UWRGRAY(.clk(Clock), .aclr(Resetn), .ena(next_wr_gray), .gray_code(Wr_data));

/*johnson_counter #(8) URDJOHN(.clk(Clock), .aclr(Resetn), .ena(read_enable), .johnson_code(rd_john));*/
/*modified from above*/
johnson_counter #(8) URDJOHN(.clk(Clock), .aclr(Resetn), .ena(read_enable && EFn), .johnson_code(rd_john));
assign next_rd_gray = read_enable && rd_john[7] && !rd_john[6] && EFn; //Johnson Code WIDTH'b1000_0000 simplified decode
/*gray_counter #(WIDTH) URDGRAY(.clk(Clock), .aclr(Resetn), .ena(next_rd_gray), .gray_code(check_data));*/
// following modified from above
gray_counter #(WIDTH) URDGRAY(.clk(Clock), .aclr(Resetn), .ena(next_rd_gray), .gray_code(check_data_d));
// modified: the following always was added
always @(posedge Clock or negedge Resetn) 
begin
  if (!Resetn)
    check_data = 0;
  else
    if (read_enable == 1'b1 && EFn == 1'b1) check_data = check_data_d;  
end 

always @(posedge Clock or negedge Resetn)
begin: URST
  if (!Resetn)
   reset_delay <= 5'b0;
  else
   reset_delay <= {reset_delay[3:0], 1'b1};       // shift left 1 place; lsb = 1
end // URST;

assign resetn_sync = reset_delay[2];

assign Wrn_next = !wr_enable_next;

DFP1B UWRO(.D(Wrn_next), .CLK(Clock), .PRE(resetn_sync), .Q(Wrn));

assign Rdn_next = !rd_enable_next;
DFP1B URDO(.D(Rdn_next), .CLK(Clock), .PRE(resetn_sync), .Q(Rdn));

assign load_value = 1'b0;
assign load_write_count = (start_test | test_done);

//
// Count the number of writes for a given test
//
/*count5e1l1 UWC(.Data(load_value), .Enable(write_enable), .Sload(load_write_count), .Aclr(resetn_sync),*/
/*               .Clock(Clock), .Q(write_count));*/
//following two lines modified from above two
countload #(COUNT_WIDTH) ucountload(.Data(load_value), .Enable(write_enable), .Sload(load_write_count), .Aclr(resetn_sync), .Clock(Clock), .Q(write_count));
/*assign write_count = 5'b00000;*/
assign burst_load = 3'b001;

//
// Write Burst counter counts up as long as write is asserted
// synchronously reloads to 0 if write is de-asserted.
//
count3e1l0 UWB(.Data(burst_load),.Enable(write_enable),.Sload(write_enable),
               .Aclr(resetn_sync),.Clock(Clock),.Q(WRITE_BURST_cnt));
//
// Read Burst counter counts up as long as Read is asserted
// synchronously reloads to 0 if Read is de-asserted.
//
count3e1l0 URB(.Data(burst_load),.Enable(read_enable),.Sload(read_enable),
               .Aclr(resetn_sync),.Clock(Clock),.Q(READ_BURST_cnt));
//
// The T1, T2, T3 signals are used when the write or Read mode is every third clock cycle.
//
always @(posedge Clock or negedge resetn_sync) begin: UTTIME
  if (!resetn_sync) begin
   T1 <= 1'b1;
   T2 <= 1'b0;
   T3 <= 1'b0;
  end
  else begin
   T1 <= T3;
   T2 <= T1;
   T3 <= T2;
  end 
end //UTTIME;

//*** The following was changed from dan's design.  The watchdog was also
// changed.  The reason is that are fifos are much larger than the ones
// dan used so a much larger value had be used before timeout occured.
//
// Removed watchdog for debug
/*watchdog #(20'hfffff) UWRNWD(.WDTIMEOUT(write_timeout),.ENABLEn(write_enable),.Clock(Clock),.Resetn(resetn_sync));*/
/*watchdog #(20'hfffff) URDNWD(.WDTIMEOUT(read_timeout),.ENABLEn(read_enable),.Clock(Clock),.Resetn(resetn_sync));*/
//
// The T1, T2, and T3 signals are one hot, should more than one or none of them be set force a reset.
//
/*wire data_err = read_enable && check_data!=Rd_data;*/
// following modified from above
wire data_err = check_data!=Rd_data;
wire t_time_err =  (T1 & T2) 
                || (T1 & T3) 
 	        || (T2 & T3) 
 	        || (!T1 & !T2 & !T3) 
	         ;
always @(posedge Clock or negedge Resetn) begin: UTERR
 if (Resetn==1'b0)
   test_reg <= 0;
 else
  test_reg <=!( t_time_err
/* 	    || write_timeout */
/* 	    || read_timeout */
 	    || data_err
            );
end //UTERR;

always @(posedge Clock or negedge resetn_sync) begin: USEQ
 if (resetn_sync==1'b0) begin
  start_test <= 1'b0;
  write_enable <= 1'b0;
  wbursting <= 1'b0;
  read_enable <= 1'b0;
  rbursting <= 1'b0;
  write_done <= 1'b0;
 end
 else begin
  start_test <=  (~reset_delay[4] &
                  reset_delay[3]);
  write_enable <= wr_enable_next;
/*// following is modified from above*/
/*  write_enable <= wr_enable_next && FFn; */
  wbursting <= wbursting_next;
  read_enable  <= rd_enable_next;
// following is modified from above
/*  read_enable  <= rd_enable_next && EFn;*/
  rbursting <= rbursting_next;
/*  if (((write_enable == 1'b1) && (write_count == { COUNT_WIDTH - 1 {1'b0}} )) || ((write_done == 1'b1) && (test_done == 1'b0))) */
  if (((write_enable == 1'b1) && (write_count == (8 + ({1'b1, { COUNT_WIDTH - 1 {1'b0}}})) )) || ((write_done == 1'b1) && (test_done == 1'b0))) 
   write_done <= 1'b1;
  else 
   write_done <= 1'b0;
 end
end //USEQ;

always @(posedge Clock or negedge resetn_sync) begin: UTEST
 if (resetn_sync==1'b0) begin
  test_done <= 1'b0;
  write_mode <= DISABLE_WRITE;
  read_mode <= READ_IF_DATA;
  test_state <= Test1;
  En_write_till_full <= 1'b0;
  En_read_till_empty <= 1'b0;
 end
 else begin
  test_done <= !test_done & write_done & !EFn; // has to be one cycle pulse
                                               // takes 1 cycle for write-done to clear

  if ((write_mode==WRITE_TILL_FULL & !EFn) | //wait till empty to start
      (write_mode==WRITE_TILL_FULL & En_write_till_full & FFn)) //go till full
   En_write_till_full <= 1'b1;
  else
   En_write_till_full <= 1'b0;

  if ((read_mode==READ_TILL_EMPTY & !FFn) | //wait till full to start reading
      (read_mode==READ_TILL_EMPTY & En_read_till_empty & EFn)) //go till empty
   En_read_till_empty <= 1'b1;
  else
   En_read_till_empty <= 1'b0;

  case (test_state)
   Test1:   begin
             write_mode <= WRITE_ONE_THIRD;
             read_mode  <= READ_BURST;
             if (test_done) begin
              write_mode <= WRITE_BURST;
              read_mode  <= READ_ONE_THIRD;
              test_state <= Test2;
             end
            end
   Test2:   begin
             write_mode <= WRITE_BURST;
             read_mode  <= READ_ONE_THIRD;
             if (test_done) begin
              write_mode <= WRITE_TILL_FULL;
              read_mode <= READ_TILL_EMPTY; 
              test_state <= Test3;
             end
            end
   Test3:   begin
             write_mode <= WRITE_TILL_FULL;
             read_mode <= READ_TILL_EMPTY; 
             if (test_done) begin
              write_mode <= WRITE_BURST;
              read_mode <= READ_BURST;
              test_state <= Test4;
             end
            end
   Test4:   begin
             write_mode <= WRITE_BURST;
             read_mode <= READ_BURST;
             if (test_done) begin
              write_mode <= WRITE_ONE_THIRD;
              read_mode  <= READ_BURST;
              test_state <= Test1;
             end
            end
   default: begin
            write_mode <= 'bx;
            read_mode <= 'bx;
           end
  endcase
 end
end // UTEST;

always @(write_mode or write_enable or write_done or WRITE_BURST_cnt or wbursting or T1 or
         En_write_till_full or AFn or FFn)
 begin: UWRITEC
  wbursting_next <= 1'b0;
  if (write_done | !FFn)
   wr_enable_next <= 1'b0; 
  else
   case (write_mode)
    WRITE_TILL_FULL: if((En_write_till_full==1'b1 && AFn==1'b1) || (En_write_till_full==1'b1 && write_enable==1'b0))
                      wr_enable_next <= 1'b1;
                     else 
                      wr_enable_next <= 1'b0;

    WRITE_ONE_THIRD: if (T1==1'b1)
                      wr_enable_next <= 1'b1;
                     else
                      wr_enable_next <= 1'b0;

    WRITE_BURST:     if ((AFn ==1'b1 || wbursting==1'b1) && WRITE_BURST_cnt[2]==1'b0) begin
                      wr_enable_next <= 1'b1;
                      wbursting_next <= 1'b1;
                     end
                     else begin
                      wr_enable_next <= 1'b0;
                      wbursting_next <= 1'b0;
                     end

    DISABLE_WRITE:   wr_enable_next <= 1'b0;

    default:         begin
                      wr_enable_next <= 'bx;
                      wbursting_next <= 1'bx;
                     end
   endcase
 end // UWRITEC;

always @(read_mode or read_enable or READ_BURST_cnt or rbursting or write_done or
        T1 or En_read_till_empty or AEn or EFn)
 begin: UREADC
  rbursting_next <= 1'b0;
  if (EFn==1'b0)
   rd_enable_next <= 1'b0; 
/*  else*/
  // following three lines are modified from above
  else if (write_done == 1'b1)
   rd_enable_next <= 1'b1;
  else
   case (read_mode)
    READ_TILL_EMPTY: if((En_read_till_empty==1'b1 && AEn==1'b1) || (En_read_till_empty==1'b1 && read_enable==1'b0))
                      rd_enable_next <= 1'b1;
                     else
                      rd_enable_next <= 1'b0;
        
    READ_ONE_THIRD:  if(T1==1'b1) rd_enable_next <= 1'b1;
                     else rd_enable_next <= 1'b0;
    
    READ_BURST:      if((AEn==1'b1 || rbursting==1'b1) && READ_BURST_cnt[2]==1'b0) begin
                      rd_enable_next <= 1'b1;
                      rbursting_next <= 1'b1;
                     end
                     else if (write_done == 1'b1 && read_enable == 1'b0) begin
                      rd_enable_next <=1'b1;
                      rbursting_next <= 1'b0;
                     end
                     else begin
                      rd_enable_next <= 1'b0;
                      rbursting_next <= 1'b0;
                     end

    READ_IF_DATA:    if (read_enable==1'b0) rd_enable_next <= 1'b1;
                     else rd_enable_next <= 1'b0;
    default:         begin
                      rd_enable_next <=1'bx;
                      rbursting_next <= 1'bx;
                     end
   endcase

 end //UREADC;

always @(negedge test_reg or negedge Resetn)
 if (Resetn==1'b0)
  Test_ok <= 1'b1;
 else
  Test_ok <= 1'b0;

endmodule
